@@ -16,6 +16,7 @@ from UM.Mesh.MeshBuilder import MeshBuilder
 from UM.Message import Message
 from UM.Platform import Platform
 from UM.PluginRegistry import PluginRegistry
+from UM.Qt.QtApplication import QtApplication
 from UM.Resources import Resources
 from UM.Scene.Iterator.DepthFirstIterator import DepthFirstIterator
 
@@ -26,8 +27,8 @@ from UM.View.GL.OpenGL import OpenGL
 from UM.View.GL.OpenGLContext import OpenGLContext
 from UM.View.GL.ShaderProgram import ShaderProgram
 
-from UM.View.View import View
 from UM.i18n import i18nCatalog
+from cura.CuraView import CuraView
 from cura.Scene.ConvexHullNode import ConvexHullNode
 from cura.CuraApplication import CuraApplication
 
@@ -48,15 +49,15 @@ catalog = i18nCatalog("cura")
 
 
 ## View used to display g-code paths.
-class SimulationView(View):
+class SimulationView(CuraView):
     # Must match SimulationView.qml
     LAYER_VIEW_TYPE_MATERIAL_TYPE = 0
     LAYER_VIEW_TYPE_LINE_TYPE = 1
     LAYER_VIEW_TYPE_FEEDRATE = 2
     LAYER_VIEW_TYPE_THICKNESS = 3
 
-    def __init__(self) -> None:
-        super().__init__()
+    def __init__(self, parent = None) -> None:
+        super().__init__(parent)
 
         self._max_layers = 0
         self._current_layer_num = 0
@@ -113,6 +114,16 @@ class SimulationView(View):
         self._wireprint_warning_message = Message(catalog.i18nc("@info:status", "Cura does not accurately display layers when Wire Printing is enabled"),
                                                   title = catalog.i18nc("@info:title", "Simulation View"))
 
+        QtApplication.getInstance().engineCreatedSignal.connect(self._onEngineCreated)
+
+    def _onEngineCreated(self) -> None:
+        plugin_path = PluginRegistry.getInstance().getPluginPath(self.getPluginId())
+        if plugin_path:
+            self.addDisplayComponent("main", os.path.join(plugin_path, "SimulationViewMainComponent.qml"))
+            self.addDisplayComponent("menu", os.path.join(plugin_path, "SimulationViewMenuComponent.qml"))
+        else:
+            Logger.log("e", "Unable to find the path for %s", self.getPluginId())
+
     def _evaluateCompatibilityMode(self) -> bool:
         return OpenGLContext.isLegacyOpenGL() or bool(Application.getInstance().getPreferences().getValue("view/force_layer_view_compatibility_mode"))
 
@@ -331,12 +342,16 @@ class SimulationView(View):
         return self._extruder_count
 
     def getMinFeedrate(self) -> float:
+        if abs(self._min_feedrate - sys.float_info.max) < 10: # Some lenience due to floating point rounding.
+            return 0.0 # If it's still max-float, there are no measurements. Use 0 then.
         return self._min_feedrate
 
     def getMaxFeedrate(self) -> float:
         return self._max_feedrate
 
     def getMinThickness(self) -> float:
+        if abs(self._min_thickness - sys.float_info.max) < 10: # Some lenience due to floating point rounding.
+            return 0.0 # If it's still max-float, there are no measurements. Use 0 then.
         return self._min_thickness
 
     def getMaxThickness(self) -> float:
