@@ -1,24 +1,34 @@
-# Copyright (c) 2017 Ultimaker B.V.
+# Copyright (c) 2019 Ultimaker B.V.
 # Cura is released under the terms of the LGPLv3 or higher.
+import json
+from queue import Queue
+from threading import Event, Thread
+from time import time
+import os
 
-from UM.OutputDevice.OutputDevicePlugin import OutputDevicePlugin
+from zeroconf import Zeroconf, ServiceBrowser, ServiceStateChange, ServiceInfo
+from PyQt5.QtNetwork import QNetworkRequest, QNetworkAccessManager
+from PyQt5.QtCore import pyqtSlot, QUrl, pyqtSignal, pyqtProperty, QObject
+from PyQt5.QtGui import QDesktopServices
+
+from cura.CuraApplication import CuraApplication
+from cura.PrinterOutputDevice import ConnectionType
+from cura.Settings.GlobalStack import GlobalStack # typing
+
+from UM.i18n import i18nCatalog
 from UM.Logger import Logger
-from UM.Application import Application
+from UM.Message import Message
+from UM.OutputDevice.OutputDevicePlugin import OutputDevicePlugin
+from UM.PluginRegistry import PluginRegistry
 from UM.Signal import Signal, signalemitter
 from UM.Version import Version
 
 from . import ClusterUM3OutputDevice, LegacyUM3OutputDevice
+from .Cloud.CloudOutputDeviceManager import CloudOutputDeviceManager
 
-from PyQt5.QtNetwork import QNetworkRequest, QNetworkAccessManager
-from PyQt5.QtCore import QUrl
-
-from zeroconf import Zeroconf, ServiceBrowser, ServiceStateChange, ServiceInfo
-from queue import Queue
-from threading import Event, Thread
-from time import time
-
-import json
+from typing import Optional
 
+i18n_catalog = i18nCatalog("cura")
 
 ##      This plugin handles the connection detection & creation of output device objects for the UM3 printer.
 #       Zero-Conf is used to detect printers, which are saved in a dict.
@@ -28,17 +38,24 @@ class UM3OutputDevicePlugin(OutputDevicePlugin):
     addDeviceSignal = Signal()
     removeDeviceSignal = Signal()
     discoveredDevicesChanged = Signal()
+    cloudFlowIsPossible = Signal()
 
     def __init__(self):
         super().__init__()
+        
         self._zero_conf = None
         self._zero_conf_browser = None
 
+        self._application = CuraApplication.getInstance()
+
+        # Create a cloud output device manager that abstracts all cloud connection logic away.
+        self._cloud_output_device_manager = CloudOutputDeviceManager()
+
         # Because the model needs to be created in the same thread as the QMLEngine, we use a signal.
         self.addDeviceSignal.connect(self._onAddDevice)
         self.removeDeviceSignal.connect(self._onRemoveDevice)
 
-        Application.getInstance().globalContainerStackChanged.connect(self.reCheckConnections)
+        self._application.globalContainerStackChanged.connect(self.reCheckConnections)
 
         self._discovered_devices = {}
         
@@ -46,6 +63,7 @@ class UM3OutputDevicePlugin(OutputDevicePlugin):
         self._network_manager.finished.connect(self._onNetworkRequestFinished)
 
         self._min_cluster_version = Version("4.0.0")
+        self._min_cloud_version = Version("5.2.0")
 
         self._api_version = "1"
         self._api_prefix = "/api/v" + self._api_version + "/"
@@ -53,7 +71,7 @@ class UM3OutputDevicePlugin(OutputDevicePlugin):
         self._cluster_api_prefix = "/cluster-api/v" + self._cluster_api_version + "/"
 
         # Get list of manual instances from preferences
-        self._preferences = Application.getInstance().getPreferences()
+        self._preferences = CuraApplication.getInstance().getPreferences()
         self._preferences.addPreference("um3networkprinting/manual_instances",
                                         "")  # A comma-separated list of ip adresses or hostnames
 
@@ -71,6 +89,26 @@ class UM3OutputDevicePlugin(OutputDevicePlugin):
         self._service_changed_request_thread = Thread(target=self._handleOnServiceChangedRequests, daemon=True)
         self._service_changed_request_thread.start()
 
+        self._account = self._application.getCuraAPI().account
+
+        # Check if cloud flow is possible when user logs in
+        self._account.loginStateChanged.connect(self.checkCloudFlowIsPossible)
+
+        # Check if cloud flow is possible when user switches machines
+        self._application.globalContainerStackChanged.connect(self._onMachineSwitched)
+
+        # Listen for when cloud flow is possible 
+        self.cloudFlowIsPossible.connect(self._onCloudFlowPossible)
+
+        # Listen if cloud cluster was added
+        self._cloud_output_device_manager.addedCloudCluster.connect(self._onCloudPrintingConfigured)
+
+        # Listen if cloud cluster was removed
+        self._cloud_output_device_manager.removedCloudCluster.connect(self.checkCloudFlowIsPossible)
+
+        self._start_cloud_flow_message = None # type: Optional[Message]
+        self._cloud_flow_complete_message = None # type: Optional[Message]
+
     def getDiscoveredDevices(self):
         return self._discovered_devices
 
@@ -83,6 +121,7 @@ class UM3OutputDevicePlugin(OutputDevicePlugin):
     ##  Start looking for devices on network.
     def start(self):
         self.startDiscovery()
+        self._cloud_output_device_manager.start()
 
     def startDiscovery(self):
         self.stop()
@@ -104,7 +143,7 @@ class UM3OutputDevicePlugin(OutputDevicePlugin):
         self.resetLastManualDevice()
 
     def reCheckConnections(self):
-        active_machine = Application.getInstance().getGlobalContainerStack()
+        active_machine = CuraApplication.getInstance().getGlobalContainerStack()
         if not active_machine:
             return
 
@@ -114,6 +153,8 @@ class UM3OutputDevicePlugin(OutputDevicePlugin):
             if key == um_network_key:
                 if not self._discovered_devices[key].isConnected():
                     Logger.log("d", "Attempting to connect with [%s]" % key)
+                    # It should already be set, but if it actually connects we know for sure it's supported!
+                    active_machine.addConfiguredConnectionType(self._discovered_devices[key].connectionType.value)
                     self._discovered_devices[key].connect()
                     self._discovered_devices[key].connectionStateChanged.connect(self._onDeviceConnectionStateChanged)
                 else:
@@ -129,9 +170,10 @@ class UM3OutputDevicePlugin(OutputDevicePlugin):
             return
         if self._discovered_devices[key].isConnected():
             # Sometimes the status changes after changing the global container and maybe the device doesn't belong to this machine
-            um_network_key = Application.getInstance().getGlobalContainerStack().getMetaDataEntry("um_network_key")
+            um_network_key = CuraApplication.getInstance().getGlobalContainerStack().getMetaDataEntry("um_network_key")
             if key == um_network_key:
                 self.getOutputDeviceManager().addOutputDevice(self._discovered_devices[key])
+                self.checkCloudFlowIsPossible()
         else:
             self.getOutputDeviceManager().removeOutputDevice(key)
 
@@ -139,6 +181,7 @@ class UM3OutputDevicePlugin(OutputDevicePlugin):
         if self._zero_conf is not None:
             Logger.log("d", "zeroconf close...")
             self._zero_conf.close()
+        self._cloud_output_device_manager.stop()
 
     def removeManualDevice(self, key, address = None):
         if key in self._discovered_devices:
@@ -238,7 +281,7 @@ class UM3OutputDevicePlugin(OutputDevicePlugin):
                 properties = device.getProperties().copy()
                 if b"incomplete" in properties:
                     del properties[b"incomplete"]
-                properties[b'cluster_size'] = len(cluster_printers_list)
+                properties[b"cluster_size"] = len(cluster_printers_list)
                 self._onRemoveDevice(instance_name)
                 self._onAddDevice(instance_name, address, properties)
 
@@ -281,8 +324,10 @@ class UM3OutputDevicePlugin(OutputDevicePlugin):
         self._discovered_devices[device.getId()] = device
         self.discoveredDevicesChanged.emit()
 
-        global_container_stack = Application.getInstance().getGlobalContainerStack()
+        global_container_stack = CuraApplication.getInstance().getGlobalContainerStack()
         if global_container_stack and device.getId() == global_container_stack.getMetaDataEntry("um_network_key"):
+            # Ensure that the configured connection type is set.
+            global_container_stack.addConfiguredConnectionType(device.connectionType.value)
             device.connect()
             device.connectionStateChanged.connect(self._onDeviceConnectionStateChanged)
 
@@ -299,7 +344,7 @@ class UM3OutputDevicePlugin(OutputDevicePlugin):
             self._service_changed_request_event.wait(timeout = 5.0)
 
             # Stop if the application is shutting down
-            if Application.getInstance().isShuttingDown():
+            if CuraApplication.getInstance().isShuttingDown():
                 return
 
             self._service_changed_request_event.clear()
@@ -325,13 +370,12 @@ class UM3OutputDevicePlugin(OutputDevicePlugin):
 
     ##  Handler for zeroConf detection.
     #   Return True or False indicating if the process succeeded.
-    #   Note that this function can take over 3 seconds to complete. Be carefull calling it from the main thread.
+    #   Note that this function can take over 3 seconds to complete. Be careful
+    #   calling it from the main thread.
     def _onServiceChanged(self, zero_conf, service_type, name, state_change):
         if state_change == ServiceStateChange.Added:
-            Logger.log("d", "Bonjour service added: %s" % name)
-
             # First try getting info from zero-conf cache
-            info = ServiceInfo(service_type, name, properties={})
+            info = ServiceInfo(service_type, name, properties = {})
             for record in zero_conf.cache.entries_with_name(name.lower()):
                 info.update_record(zero_conf, time(), record)
 
@@ -342,7 +386,6 @@ class UM3OutputDevicePlugin(OutputDevicePlugin):
 
             # Request more data if info is not complete
             if not info.address:
-                Logger.log("d", "Trying to get address of %s", name)
                 info = zero_conf.get_service_info(service_type, name)
 
             if info:
@@ -362,4 +405,135 @@ class UM3OutputDevicePlugin(OutputDevicePlugin):
             Logger.log("d", "Bonjour service removed: %s" % name)
             self.removeDeviceSignal.emit(str(name))
 
-        return True
\ No newline at end of file
+        return True
+
+    ## Check if the prerequsites are in place to start the cloud flow
+    def checkCloudFlowIsPossible(self) -> None:
+        Logger.log("d", "Checking if cloud connection is possible...")
+
+        # Pre-Check: Skip if active machine already has been cloud connected or you said don't ask again
+        active_machine = self._application.getMachineManager().activeMachine # type: Optional["GlobalStack"]
+        if active_machine:
+            
+            # Check 1A: Printer isn't already configured for cloud
+            if ConnectionType.CloudConnection.value in active_machine.configuredConnectionTypes:
+                Logger.log("d", "Active machine was already configured for cloud.")
+                return
+            
+            # Check 1B: Printer isn't already configured for cloud
+            if active_machine.getMetaDataEntry("cloud_flow_complete", False):
+                Logger.log("d", "Active machine was already configured for cloud.")
+                return
+
+            # Check 2: User did not already say "Don't ask me again"
+            if active_machine.getMetaDataEntry("do_not_show_cloud_message", False):
+                Logger.log("d", "Active machine shouldn't ask about cloud anymore.")
+                return
+        
+            # Check 3: User is logged in with an Ultimaker account
+            if not self._account.isLoggedIn:
+                Logger.log("d", "Cloud Flow not possible: User not logged in!")
+                return
+
+            # Check 4: Machine is configured for network connectivity
+            if not self._application.getMachineManager().activeMachineHasNetworkConnection:
+                Logger.log("d", "Cloud Flow not possible: Machine is not connected!")
+                return
+            
+            # Check 5: Machine has correct firmware version
+            firmware_version = self._application.getMachineManager().activeMachineFirmwareVersion # type: str
+            if not Version(firmware_version) > self._min_cloud_version:
+                Logger.log("d", "Cloud Flow not possible: Machine firmware (%s) is too low! (Requires version %s)",
+                                firmware_version,
+                                self._min_cloud_version)
+                return
+            
+            Logger.log("d", "Cloud flow is possible!")
+            self.cloudFlowIsPossible.emit()
+
+    def _onCloudFlowPossible(self) -> None:
+        # Cloud flow is possible, so show the message
+        if not self._start_cloud_flow_message:
+            self._createCloudFlowStartMessage()
+        if self._start_cloud_flow_message and not self._start_cloud_flow_message.visible:
+            self._start_cloud_flow_message.show()        
+
+    def _onCloudPrintingConfigured(self) -> None:
+        # Hide the cloud flow start message if it was hanging around already
+        # For example: if the user already had the browser openen and made the association themselves
+        if self._start_cloud_flow_message and self._start_cloud_flow_message.visible:
+            self._start_cloud_flow_message.hide()
+        
+        # Cloud flow is complete, so show the message
+        if not self._cloud_flow_complete_message:
+            self._createCloudFlowCompleteMessage()
+        if self._cloud_flow_complete_message and not self._cloud_flow_complete_message.visible:
+            self._cloud_flow_complete_message.show()
+        
+        # Set the machine's cloud flow as complete so we don't ask the user again and again for cloud connected printers
+        active_machine = self._application.getMachineManager().activeMachine
+        if active_machine:
+            active_machine.setMetaDataEntry("do_not_show_cloud_message", True)
+        return
+
+    def _onDontAskMeAgain(self, checked: bool) -> None:
+        active_machine = self._application.getMachineManager().activeMachine # type: Optional["GlobalStack"]
+        if active_machine:
+            active_machine.setMetaDataEntry("do_not_show_cloud_message", checked)
+            if checked:
+                Logger.log("d", "Will not ask the user again to cloud connect for current printer.")
+        return
+
+    def _onCloudFlowStarted(self, messageId: str, actionId: str) -> None:
+        address = self._application.getMachineManager().activeMachineAddress # type: str
+        if address:
+            QDesktopServices.openUrl(QUrl("http://" + address + "/cloud_connect"))
+            if self._start_cloud_flow_message:
+                self._start_cloud_flow_message.hide()
+                self._start_cloud_flow_message = None
+        return
+    
+    def _onReviewCloudConnection(self, messageId: str, actionId: str) -> None:
+        address = self._application.getMachineManager().activeMachineAddress # type: str
+        if address:
+            QDesktopServices.openUrl(QUrl("http://" + address + "/settings"))
+        return
+
+    def _onMachineSwitched(self) -> None:
+        # Hide any left over messages
+        if self._start_cloud_flow_message is not None and self._start_cloud_flow_message.visible:
+            self._start_cloud_flow_message.hide()
+        if self._cloud_flow_complete_message is not None and self._cloud_flow_complete_message.visible:
+            self._cloud_flow_complete_message.hide()
+
+        # Check for cloud flow again with newly selected machine
+        self.checkCloudFlowIsPossible()
+
+    def _createCloudFlowStartMessage(self):
+        self._start_cloud_flow_message = Message(
+            text = i18n_catalog.i18nc("@info:status", "Send and monitor print jobs from anywhere using your Ultimaker account."),
+            lifetime = 0,
+            image_source = QUrl.fromLocalFile(os.path.join(
+                PluginRegistry.getInstance().getPluginPath("UM3NetworkPrinting"),
+                "resources", "svg", "cloud-flow-start.svg"
+            )),
+            image_caption = i18n_catalog.i18nc("@info:status", "Connect to Ultimaker Cloud"),
+            option_text = i18n_catalog.i18nc("@action", "Don't ask me again for this printer."),
+            option_state = False
+        )
+        self._start_cloud_flow_message.addAction("", i18n_catalog.i18nc("@action", "Get started"), "", "")
+        self._start_cloud_flow_message.optionToggled.connect(self._onDontAskMeAgain)
+        self._start_cloud_flow_message.actionTriggered.connect(self._onCloudFlowStarted)
+
+    def _createCloudFlowCompleteMessage(self):
+        self._cloud_flow_complete_message = Message(
+            text = i18n_catalog.i18nc("@info:status", "You can now send and monitor print jobs from anywhere using your Ultimaker account."),
+            lifetime = 30,
+            image_source = QUrl.fromLocalFile(os.path.join(
+                PluginRegistry.getInstance().getPluginPath("UM3NetworkPrinting"),
+                "resources", "svg", "cloud-flow-completed.svg"
+            )),
+            image_caption = i18n_catalog.i18nc("@info:status", "Connected!")
+        )
+        self._cloud_flow_complete_message.addAction("", i18n_catalog.i18nc("@action", "Review your connection"), "", "", 1) # TODO: Icon
+        self._cloud_flow_complete_message.actionTriggered.connect(self._onReviewCloudConnection)
\ No newline at end of file
