@@ -11,10 +11,7 @@ class GenericMaterialsModel(BaseMaterialsModel):
         self._update()
 
     def _update(self):
-
-        # Perform standard check and reset if the check fails
         if not self._canUpdate():
-            self.setItems([])
             return
 
         # Get updated list of favorites
