@@ -1,10 +1,11 @@
 # Copyright (c) 2018 Ultimaker B.V.
 # Cura is released under the terms of the LGPLv3 or higher.
+from datetime import datetime
 import json
 import random
 from hashlib import sha512
 from base64 import b64encode
-from typing import Dict, Optional
+from typing import Optional
 
 import requests
 
@@ -13,6 +14,9 @@ from UM.Logger import Logger
 from cura.OAuth2.Models import AuthenticationResponse, UserProfile, OAuth2Settings
 
 
+TOKEN_TIMESTAMP_FORMAT = "%Y-%m-%d %H:%M:%S"
+
+
 #   Class containing several helpers to deal with the authorization flow.
 class AuthorizationHelpers:
     def __init__(self, settings: "OAuth2Settings") -> None:
@@ -37,7 +41,10 @@ class AuthorizationHelpers:
             "code_verifier": verification_code,
             "scope": self._settings.CLIENT_SCOPES if self._settings.CLIENT_SCOPES is not None else "",
             }
-        return self.parseTokenResponse(requests.post(self._token_url, data = data))  # type: ignore
+        try:
+            return self.parseTokenResponse(requests.post(self._token_url, data = data))  # type: ignore
+        except requests.exceptions.ConnectionError:
+            return AuthenticationResponse(success=False, err_message="Unable to connect to remote server")
 
     #   Request the access token from the authorization server using a refresh token.
     #   \param refresh_token:
@@ -50,7 +57,10 @@ class AuthorizationHelpers:
             "refresh_token": refresh_token,
             "scope": self._settings.CLIENT_SCOPES if self._settings.CLIENT_SCOPES is not None else "",
         }
-        return self.parseTokenResponse(requests.post(self._token_url, data = data))  # type: ignore
+        try:
+            return self.parseTokenResponse(requests.post(self._token_url, data = data))  # type: ignore
+        except requests.exceptions.ConnectionError:
+            return AuthenticationResponse(success=False, err_message="Unable to connect to remote server")
 
     @staticmethod
     #   Parse the token response from the authorization server into an AuthenticationResponse object.
@@ -75,15 +85,21 @@ class AuthorizationHelpers:
                                       access_token=token_data["access_token"],
                                       refresh_token=token_data["refresh_token"],
                                       expires_in=token_data["expires_in"],
-                                      scope=token_data["scope"])
+                                      scope=token_data["scope"],
+                                      received_at=datetime.now().strftime(TOKEN_TIMESTAMP_FORMAT))
 
     #   Calls the authentication API endpoint to get the token data.
     #   \param access_token: The encoded JWT token.
     #   \return: Dict containing some profile data.
     def parseJWT(self, access_token: str) -> Optional["UserProfile"]:
-        token_request = requests.get("{}/check-token".format(self._settings.OAUTH_SERVER_URL), headers = {
-            "Authorization": "Bearer {}".format(access_token)
-        })
+        try:
+            token_request = requests.get("{}/check-token".format(self._settings.OAUTH_SERVER_URL), headers = {
+                "Authorization": "Bearer {}".format(access_token)
+            })
+        except requests.exceptions.ConnectionError:
+            # Connection was suddenly dropped. Nothing we can do about that.
+            Logger.log("w", "Something failed while attempting to parse the JWT token")
+            return None
         if token_request.status_code not in (200, 201):
             Logger.log("w", "Could not retrieve token data from auth server: %s", token_request.text)
             return None
